/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef NPP_REMOTEDPPHISTOGRAM_H
#define NPP_REMOTEDPPHISTOGRAM_H

#include "RemoteDPPLiveProcess.h"
#include <boost/thread.hpp>
#include <stdint.h>
#include <memory>

namespace npp {

class RemoteDPPHistogram : public RemoteDPPLiveProcess {

public:
	//! Type of controller
	static const std::string TYPE;

	RemoteDPPHistogram(const std::string& name);
	RemoteDPPHistogram(const RemoteDPPHistogram& controller);
	virtual ~RemoteDPPHistogram();

	virtual void postConfiguration();
	virtual void setDetectorControllers(const std::vector<ControllerPtr<acquisition::DetectorController> > & detectorControllers);

	virtual void start();
	virtual void raz();

	void updateListModeData(int index);

private:
	void resetData();

	Property<int32> triggerCrate;
	Property<int32> triggerBoard;
	Property<int32> triggerChannel;
	Property<int32> delay;
	Property<int32> dT;
	Property<int32> maxBlock;
	Property<int32> refreshTimeS;

	Property<int32> totalNumberOfChannels;

	DynamicProperty<int32> crateId;
	DynamicProperty<int32> boardId;
	DynamicProperty<int32> channelId;
	DynamicProperty<float64> channelRate;

	DynamicArrayProperty<int32> xData;
	DynamicArrayProperty<int32> histogram;
	DynamicProperty<std::string> histogramName;
	DynamicProperty<std::string> xLegend;
	DynamicProperty<std::string> yLegend;

	std::auto_ptr<cameo::application::Instance> m_histoApplication;
};

}

#endif
