/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "RealCAENSYx527Driver.h"
#include "CAENSYx527Driver.h"
#include "CAENSYx527Def.h"
#include "CAENHVWrapper.h"

#include <iostream>

using namespace std;

namespace caen_syx527 {

/*
 * Constructor
 */
RealCAENSYx527Driver::RealCAENSYx527Driver(CAENSYx527Driver* owner) :
	CAENSYx527State(owner),
	m_oldValue(0) {


}

/*
 * Destructor
 */
RealCAENSYx527Driver::~RealCAENSYx527Driver() {

	// Empty
}

/*
 * init
 */
void RealCAENSYx527Driver::init() {
		//char address[30], userName[30], passwd[30];
		int i, j,itotchannel=0,ntotchannel=0, link=0;
		int sysHndl=-1;
		CAENHV_SYSTEM_TYPE_t sysType;
		CAENHVRESULT  ret;
		string temp;
		unsigned short	NrOfSl, *SerNumList, *NrOfCh;
		char			*ModelList, *DescriptionList;
		unsigned char	*FmwRelMinList, *FmwRelMaxList;

		sysType=owner()->model;
		if(owner()->linktype()=="TCPIP") link = LINKTYPE_TCPIP;
		else if(owner()->linktype()=="USB")link = LINKTYPE_USB;
		//strcpy(address,owner()->address().c_str());
		//strcpy(userName, "admin");
		//strcpy(passwd, "admin");
		//cout << owner()->address << endl;
		//ret = CAENHV_InitSystem(sysType, link, address, userName, passwd, &sysHndl);
		ret = CAENHV_InitSystem(sysType, link, (void *)owner()->address().c_str(), "admin", "admin", &sysHndl);

		owner()->sysHndl=sysHndl;

		if(ret == CAENHV_OK )
			{
			owner()->statusconnection=true;
		    ret = CAENHV_GetCrateMap(sysHndl, &NrOfSl, &NrOfCh, &ModelList, &DescriptionList, &SerNumList,
	                                  &FmwRelMinList, &FmwRelMaxList );
		    if(ret != CAENHV_OK ) owner()->statusconnection=false;
		    cout << "Number of slots " << NrOfSl << endl;
		    owner()->nslot= NrOfSl;
		    char	*m = ModelList, *d = DescriptionList;
		    char firmware[10];

		    for (i=0;i<NrOfSl;i++, m += strlen(m) + 1, d += strlen(d) + 1 ){
		    	owner()->nchannelperslot.set(i,NrOfCh[i]);

		    	if( *m == '\0' ){
		    		owner()->modelboard.set(i,"None");
		    		owner()->descrboard.set(i,"None");
		    		owner()->firmboard.set(i,"None");
		    	}
		    	else{
		    		owner()->modelboard.set(i,m);
			    	owner()->descrboard.set(i,d);
			    	sprintf(firmware,"%d.%d",FmwRelMaxList[i],FmwRelMinList[i]);
			    	owner()->firmboard.set(i,(string)firmware);
		    	}
		    	owner()->serialboard.set(i,(int32)SerNumList[i]);

		    	cout << "Slot  " << i << " : model = " << m;
		    	cout << " Number of channels = " << NrOfCh[i] << endl;

		    	ntotchannel+=NrOfCh[i];
		    	owner()->ntotchannel=ntotchannel;
		    	for(j=0;j<NrOfCh[i];j++){

		    		owner()->islot.set(itotchannel,i);
		    		owner()->ichannel.set(itotchannel,j);
		    		itotchannel+=1;
		    	}
		    }



		    free(SerNumList), free(ModelList), free(DescriptionList), free(FmwRelMinList),
		    		    	free(FmwRelMaxList), free(NrOfCh);

		    readParamChannels();

		}
		else{
			 owner()->nslot= 0;
			 owner()->statusconnection=false;
		}

}


/*
 * readParam
 */
void RealCAENSYx527Driver::readParamChannels() {
	CAENHVRESULT ret;
	unsigned short nch, listaCh[CAENSYX527_MAX_CHANNEL_PER_BOARD];
	char  listNameCh[CAENSYX527_MAX_CHANNEL_PER_BOARD][MAX_CH_NAME];
	float			fParValList[CAENSYX527_MAX_CHANNEL_PER_BOARD];
	int32   iParValList[CAENSYX527_MAX_CHANNEL_PER_BOARD];
	int nchanneloffset=0;
	//cout << "Slot  "  << endl;

	//loop on slots
	for(unsigned short islt=0;islt<owner()->nslot();islt++){
		nch=(unsigned short)owner()->nchannelperslot(islt);

		if(nch>0){

			for(int i = 0; i < nch; i++ ){
				listaCh[i] =(unsigned short) i;
			}
			ret = CAENHV_GetChName((int)owner()->sysHndl, islt, nch, listaCh, listNameCh);

			if( ret == CAENHV_OK){
				owner()->statusconnection=true;
				for(int i= 0; i < nch; i++ ){
					owner()->namechannel.set(nchanneloffset+i,listNameCh[i]);
				}
			}else owner()->statusconnection=false;

			ret = CAENHV_GetChParam((int)owner()->sysHndl, islt, "V0Set", nch, listaCh, fParValList);
			if( ret == CAENHV_OK){
				owner()->statusconnection=true;
				for(int i= 0; i < nch; i++ ){
					owner()->v0set.set(nchanneloffset+i,fParValList[i]);
				}
			}else owner()->statusconnection=false;

			ret = CAENHV_GetChParam((int)owner()->sysHndl, islt, "I0Set", nch, listaCh, fParValList);
			if( ret == CAENHV_OK){
				owner()->statusconnection=true;
				for(int i= 0; i < nch; i++ ){
					owner()->i0set.set(nchanneloffset+i,fParValList[i]);
				}
			}else owner()->statusconnection=false;

			ret = CAENHV_GetChParam((int)owner()->sysHndl, islt, "VMon", nch, listaCh, fParValList);
			if( ret == CAENHV_OK){
				owner()->statusconnection=true;
				for(int i= 0; i < nch; i++ ){
					owner()->vmon.set(nchanneloffset+i,fParValList[i]);
				}
			}else owner()->statusconnection=false;

			ret = CAENHV_GetChParam((int)owner()->sysHndl, islt, "IMon", nch, listaCh, fParValList);
			if( ret == CAENHV_OK){
				owner()->statusconnection=true;
				for(int i= 0; i < nch; i++ ){
					owner()->imon.set(nchanneloffset+i,fParValList[i]);
				}
			}else owner()->statusconnection=false;
			ret = CAENHV_GetChParam((int)owner()->sysHndl, islt, "RUp", nch, listaCh, fParValList);
			if( ret == CAENHV_OK){
				for(int i= 0; i < nch; i++ ){
					owner()->rampup.set(nchanneloffset+i,fParValList[i]);
				}
			}else owner()->statusconnection=false;

			ret = CAENHV_GetChParam((int)owner()->sysHndl, islt, "RDWn", nch, listaCh, fParValList);
			if( ret == CAENHV_OK){
				owner()->statusconnection=true;
				for(int i= 0; i < nch; i++ ){
					owner()->rampdown.set(nchanneloffset+i,fParValList[i]);
				}
			}else owner()->statusconnection=false;

			ret = CAENHV_GetChParam((int)owner()->sysHndl, islt, "Pw", nch, listaCh, iParValList);
			if( ret == CAENHV_OK){
				owner()->statusconnection=true;
				for(int i= 0; i < nch; i++ ){
					if(iParValList[i]!=0) owner()->power.set(nchanneloffset+i,"On");
					else  owner()->power.set(nchanneloffset+i,"Off");
				}
			}else owner()->statusconnection=false;

			ret = CAENHV_GetChParam((int)owner()->sysHndl, islt, "Status", nch, listaCh, iParValList);
			if( ret == CAENHV_OK){
				owner()->statusconnection=true;
				for(int i= 0; i < nch; i++ ){
					string str_status="";
					for(int j=9;j>=0;j--){
						if(iParValList[i]&(1<<j)){
							str_status=STATUSCHANNEL[j];
							break;
						}
					}
					owner()->statuschannel.set(nchanneloffset+i,str_status);
				}
			}else owner()->statusconnection=false;
			nchanneloffset+=nch;
		}
	}

}


/*
 * writeParam
 */
void RealCAENSYx527Driver::writeParam(int iChannel,string Param,float value) {

	unsigned short nch=1;
	unsigned short ich[1],isl;
	isl=(unsigned short)owner()->islot(iChannel);
	ich[0]=(unsigned short)owner()->ichannel(iChannel);

		if(Param=="Name"){
			//cout << "WriteParam " << Param << " = " << owner()->namechannel(i).c_str() << endl;
			CAENHV_SetChName((int)owner()->sysHndl,isl,nch,ich,owner()->namechannel(iChannel).c_str());
		}
		else{
			//cout << "WriteParam " << Param.c_str() << " = " << value << "owner()->v0set(i)="<< owner()->v0set(i) <<endl;
			CAENHV_SetChParam((int)owner()->sysHndl,isl, Param.c_str(), nch,ich,&value);
		}

}

/*
 * powerOn
 */
void RealCAENSYx527Driver::powerOn(int iChannel) {

	unsigned short nch=1;
	unsigned short ich[1],isl;
    unsigned long value=1;
	isl=(unsigned short)owner()->islot(iChannel);
	ich[0]=(unsigned short)owner()->ichannel(iChannel);
		CAENHV_SetChParam((int)owner()->sysHndl,isl, "Pw", nch,ich,&value);
		owner()->statuschannel.set(iChannel,"Up");


}

/*
 * powerOff
 */
void RealCAENSYx527Driver::powerOff(int iChannel) {

	unsigned short nch=1;
	unsigned short ich[1],isl;
	unsigned long value=0;
	isl=(unsigned short)owner()->islot(iChannel);
	ich[0]=(unsigned short)owner()->ichannel(iChannel);
		CAENHV_SetChParam((int)owner()->sysHndl,isl, "Pw", nch,ich,&value);
		owner()->statuschannel.set(iChannel,"Down");

}

}
